/*
	Author: B.McA (jdoe407)
	Date: March 06, 2005

	Purpose: Patcher for Fallout2.exe US 1.02
	Usage: Fallout2Patcher.exe
	Todo: Stuff
*/

#include <windows.h>
#include <stdarg.h>
#include "resource.h"
#include "commctrl.h"
#include "patcher.h"

#pragma comment(lib, "comctl32.lib")

LRESULT CALLBACK dlgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
HWND WINAPI CreateTrackbar(HWND hwndDlg, UINT iXcoord, UINT iYcoord, UINT iWidth, UINT iHeigth, UINT iMin, UINT iMax);
VOID WINAPI TBNotifications(WPARAM wParam, HWND hwndTrack, UINT iSelMin, UINT iSelMax);
void initPatcher();
void editPatches(patch *patchData);
void outputMessage(char *buff, ...);

HINSTANCE gHinst = NULL;
HWND gHwnd = NULL;
HWND gHwndTrack = NULL;

int gResolution = 0;
bool bSlowDown = false;
UINT gSlowDownSpeed = 40000;

int WINAPI WinMain(HINSTANCE hInst, HINSTANCE pInst, LPSTR cmdLine, int cmdShow)
{
	gHinst = hInst;
	MSG msg;

	gHwnd = CreateDialog(NULL, MAKEINTRESOURCE(IDD_Main), NULL, (DLGPROC)dlgProc);
	gHwndTrack = CreateTrackbar(gHwnd, 140, 70, 85, 50, 10, 10000);

	ShowWindow(gHwnd, cmdShow);

	while(GetMessage(&msg, 0,0,0) > 0)
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return 0;
}

/*******************************************************
*           Start of nasty message handling            *
*******************************************************/
LRESULT CALLBACK dlgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
		case WM_COMMAND:
			switch(LOWORD(wParam))
			{
				case IDC_btnQuit:
					PostQuitMessage(0);
					break;
				case IDC_btnPatch:
					initPatcher();
					break;
				case IDC_chkSlowDown:
					bSlowDown = !bSlowDown;
					EnableWindow(gHwndTrack, bSlowDown);
					break;
				case IDC_rad640x480:
					gResolution = 0;
					break;
				case IDC_rad800x600:
					gResolution = 1;
					break;
				case IDC_rad1024x768:
					gResolution = 2;
					break;
				case IDC_rad1280x1024:
					gResolution = 3;
					break;
				case IDC_rad1200x1600:
					gResolution = 4;
					break;
			}
			break;
		case WM_HSCROLL:
			{
				switch(LOWORD(wParam))
				{
					case TB_THUMBTRACK: case TB_LINEDOWN: case TB_LINEUP: case TB_THUMBPOSITION:
					{
						gSlowDownSpeed = (UINT)SendMessage(gHwndTrack, TBM_GETPOS, 0, 0)*2000;
						break;
					}
					case TB_PAGEDOWN:
					{
						gSlowDownSpeed = (UINT)SendMessage(gHwndTrack, TBM_GETPOS, 0, 0)+100;
						SendMessage(gHwndTrack, TBM_SETPOS, (WPARAM)TRUE, (LPARAM) gSlowDownSpeed);
						gSlowDownSpeed *= 2000;
						break;
					}
					case TB_PAGEUP:
					{
						gSlowDownSpeed = (UINT)SendMessage(gHwndTrack, TBM_GETPOS, 0, 0)-100;
						SendMessage(gHwndTrack, TBM_SETPOS, (WPARAM)TRUE, (LPARAM) gSlowDownSpeed);
						gSlowDownSpeed *= 2000;
						break;
					}
				}
				SetDlgItemInt(gHwnd, IDC_lblSlowDown, (LPARAM)gSlowDownSpeed, true);
				break;
			}
		case WM_INITDIALOG:
			return true;
		case WM_CLOSE:
			PostQuitMessage(0);
			break;
		default:
			return false;
	}
	return true;
}

// From MSDN's example, YEAY!
HWND WINAPI CreateTrackbar(HWND hwndDlg, UINT iXcoord, UINT iYcoord, UINT iWidth, UINT iHeigth, UINT iMin, UINT iMax)  
{ 
    InitCommonControls(); // loads common control's DLL 

    HWND hwndTrack = CreateWindowEx(0, TRACKBAR_CLASS, "Trackbar Control", WS_CHILD | WS_VISIBLE |WS_DISABLED, iXcoord, iYcoord, iWidth, iHeigth, hwndDlg, NULL, gHinst, NULL); 

    SendMessage(hwndTrack, TBM_SETRANGE,(WPARAM) TRUE, (LPARAM) MAKELONG(iMin, iMax));
    SendMessage(hwndTrack, TBM_SETPAGESIZE, 0, (LPARAM) 4); 

    SetFocus(hwndTrack); 

    return hwndTrack; 
} 

/*******************************************************
*            End of nasty message handling             *
*******************************************************/

void initPatcher()
{
	patch patchData[] =
	{
		{   // Patch 1
			0x0000bd7a,
			"\xba\xa8\x0b",
			"\xcc\xf3\x0a",
			3
		},
		{   // Patch 2
			0x0000d505,
			"\x2f\x91\x0b",
			"\x41\xdc\x0a",
			3
		},
		{   // Patch 3
			0x0002f961,
			"\xd3\x6c\x09",
			"\xe5\xb7\x08",
			3
		},
		{   // Patch 4
			0x000343b3,
			"\x81\x22\x09",
			"\x93\x6d\x08",
			3
		},
		{   // Patch 5
			0x0004dcbd,
			"\x77\x89\x07",
			"\x99\xd4\x06",
			3
		},
		{   // Patch 6
			0x00051a00,
			"\x34\x4c\x07",
			"\x66\x97\x06",
			3
		},
		{   // Patch 7
			0x00071a81,
			"\xb3\x4b\x05",
			"\xc5\x96\x04",
			3
		},
		{   // Patch 8
			0x00080406,
			"\x2e\x62\x04",
			"\x40\xad\x03",
			3
		},
		{   // Patch 9
			0x0009c661,
			"\xd3\x9f\x02",
			"\xfd\xea\x01",
			3
		},
		{   // Patch 10
			0x000a9617,
			"\x8b\x04\x9d\xbc\xdc\x51\x00",
			"\xb8\x06\xad\x4c\x00\x90\x90",
			7
		},
		{   // Patch 11
			0x000b025d,
			"\xe8\x16\x8d",
			"\xe9\x6a\xaf",
			3
		},
		{   // Patch 12
			0x000bb106,
			"\x8b\xc0\x53\x52\xbb\x08",
			"\x52\xeb\x18\xba\xe0\x01",
			6
		},
		{   // Patch 13
			0x000bb10e,
			"\x00\xba\xc8",
			"\xb8\x80\x02",
			3
		},
		{   // Patch 14
			0x000bb113,
			"\x00\xb8\x40\x01\x00",
			"\xeb\x2e\xba\x58\x02",
			5
		},
		{   // Patch 15
			0x000bb119,
			"\xe8\xfe\x00\x00",
			"\x00\xb8\x20\x03",
			4
		},
		{   // Patch 16
			0x000bb11e,
			"\x5a\x5b\xc3\x8d\x40\x00\x53\x52\xbb\x08\x00",
			"\x00\xeb\x22\xba\x00\x03\x00\x00\xb8\x00\x04",
			11
		},
		{   // Patch 17
			0x000bb12b,
			"\xba\xf0\x00",
			"\xeb\x16\xba",
			3
		},
		{   // Patch 18
			0x000bb12f,
			"\x00\xb8\x40\x01",
			"\x04\x00\x00\xb8",
			4
		},
		{   // Patch 19
			0x000bb134,
			"\x00\xe8\xe2\x00\x00\x00\x5a\x5b\xc3\x8d\x40\x00\x53\x52\xbb\x08\x00\x00",
			"\x05\x00\x00\xeb\x0a\xba\xb0\x04\x00\x00\xb8\x40\x06\x00\x00\xe8\x84\x01",
			18
		},
		{   // Patch 20
			0x000bb147,
			"\xba\x90\x01\x00\x00\xb8\x40\x01",
			"\x00\x5a\xc3\x81\xc2\x90\x00\x00",
			8
		},
		{   // Patch 21
			0x000bb150,
			"\x00\xe8\xc6",
			"\x05\xc0\x00",
			3
		},
		{   // Patch 22
			0x000bb155,
			"\x00\x5a\x5b\xc3\x8d\x40\x00\xb8\xff\xff\xff\xff\xc3\x8b\xc0\x52\xba\xe0\x01\x00\x00\xb8\x80\x02\x00",
			"\xe9\xde\xb4\x00\x00\x81\xc2\x85\x01\x00\x00\xeb\xee\x81\xc2\x20\x01\x00\x00\xeb\xe6\x81\xc2\x22\x01",
			25
		},
		{   // Patch 23
			0x000bb16f,
			"\xe8\x58\x01\x00\x00\x5a\xc3\x8b\xc0\x53\x52\xbb\x10\x00\x00\x00\xba\xe0\x01\x00\x00\xb8\x80\x02\x00\x00\xe8\x8e\x00\x00\x00\x5a\x5b\xc3\x8d\x40\x00\x52\xba",
			"\x00\xeb\xe3\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90",
			39
		},
		{   // Patch 24
			0x000bb197,
			"\x01\x00\x00\xb8\x80\x02\x00\x00\xe8\x28\x01\x00\x00\x5a\xc3\x8b\xc0\x52\xba\x58\x02\x00\x00\xb8\x20\x03\x00\x00\xe8\x14\x01\x00\x00\x5a\xc3\x8b\xc0\x52\xba\x00\x03\x00\x00\xb8\x00\x04\x00\x00\xe8\x00\x01\x00\x00\x5a\xc3\x8b\xc0\x52\xba",
			"\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x50\x33\xc0\x40\x3d\x00",
			59
		},
		{   // Patch 25
			0x000bb1d3,
			"\x04\x00\x00\xb8\x00\x05\x00\x00\xe8\xec\x00\x00\x00\x5a\xc3",
			"\x00\x01\x7c\xf8\x58\xe8\x9b\xdd\xff\xff\xe9\x80\x50\xff\xff",
			15
		},
	};

	patch customPatchData[] =
	{
		{   // Patch 1 - Custom resolution switch (see documents)
			0x000BB108,
			"\x18",
			"\x18",
			1
		},
		{   // Patch 2 - Custom resolution settings (general centering EDX)
			0x000bb14c,
			"\x90\x00\x00\x00",
			"\x90\x00\x00\x00",
			4
		},
		{   // Patch 3 - Custom resolution settings (general centering EAX)
			0x000bb151,
			"\xc0\x00\x00\x00",
			"\xc0\x00\x00\x00",
			4
		},
		{   // Patch 4 - Custom resolution settings (IFACE centering)
			0x000bb15c,
			"\x85\x01\x00\x00",
			"\x85\x01\x00\x00",
			4
		},
		{   // Patch 5 - Custom resolution settings (skilldex centering)
			0x000bb164,
			"\x20\x01\x00\x00",
			"\x20\x01\x00\x00",
			4
		},
		{   // Patch 6 - Custom resolution settings (stealth centering)
			0x000bb16c,
			"\x22\x01\x00\x00",
			"\x22\x01\x00\x00",
			4
		},
		{   // Patch 7 - Custom world map slowdown speed
			0x000bb1d1,
			"\x00\x00\x00\x01",
			"\x00\x00\x00\x01",
			4
		},
		{   // Patch 11
			0x000b025d,
			"\xe9\x6a\xaf",
			"\xe9\x6a\xaf",
			3
		},
	};


	editPatches(customPatchData);
	
	filePatcher fpMain("Fallout2.exe", patchData, sizeof(patchData)/sizeof(patch), outputMessage);

	fpMain.applyMorePatches(customPatchData, sizeof(customPatchData)/sizeof(patch));
}

void editPatches(patch *patchData)
{
	switch(gResolution)
	{
		case 0: // 640x480 - everything is already 0
			for(int i = 0; i < 6; i++)
			{
				patchData[i].patchedData = "\x00\x00\x00\x00";
			}
			break;
		case 1: // 800x600
			patchData[0].patchedData = "\x0C";
			patchData[1].patchedData = "\x3C\x00\x00\x00";
			patchData[2].patchedData = "\x50\x00\x00\x00";
			patchData[3].patchedData = "\xDD\x00\x00\x00";
			patchData[4].patchedData = "\x78\x00\x00\x00";
			patchData[5].patchedData = "\x7A\x00\x00\x00";
			break;
	//case 2: // 1024x768 - its the default, no changes needed
		//	patchData[16].patchedData[0] = '\x18';
		//	break;
		case 3: // 1280x1024
			patchData[0].patchedData = "\x24";
			patchData[1].patchedData = "\x10\x01\x00\x00";
			patchData[2].patchedData = "\x40\x01\x00\x00";
			patchData[3].patchedData = "\x85\x02\x00\x00";
			patchData[4].patchedData = "\x20\x02\x00\x00";
			patchData[5].patchedData = "\x22\x02\x00\x00";
			break;
		case 4: // 1600x1200
			patchData[0].patchedData = "\x30";
			patchData[1].patchedData = "\x68\x01\x00\x00";
			patchData[2].patchedData = "\xE0\x01\x00\x00";
			patchData[3].patchedData = "\x35\x03\x00\x00";
			patchData[4].patchedData = "\xD0\x02\x00\x00";
			patchData[5].patchedData = "\xD2\x02\x00\x00";
			break;
	}
	if(bSlowDown)
	{
		// get assembly style number from gSlowDownSpeed
		char *hexNum = numToAsm(gSlowDownSpeed);
		// convert string to patch style string (not null terminated)
		char ch = 0;
		char *hexBuffer = new char[patchData[5].length];

		for(int i = 0, x = 0; i < 7; i+=2, x++)
		{
			ch = 0;
			if(hexNum[i] >= 97 && hexNum[i] <= 102)
			{
				ch += (hexNum[i]-87)*16;
			}
			else
			{
				ch += (hexNum[i]-48)*16;
			}
			if(hexNum[i+1] >= 97 && hexNum[i+1] <= 102)
			{
				ch += (hexNum[i+1]-87);
			}
			else
			{
				ch += (hexNum[i+1]-48);
			}
			hexBuffer[x] = ch;
		}

		patchData[6].patchedData = hexBuffer;
	}
	else
	{
		// if not checked, restore the original call and never do our jump to
		//  the large loop
		patchData[7].patchedData = "\xe8\x16\x8d";
	}
}


void outputMessage(char *buff, ...)
{
	char * str;
	va_list argList;

	va_start(argList, buff);
	str = new char[_vscprintf(buff, argList)];
	vsprintf(str, buff, argList);
	va_end(argList);

	MessageBox(gHwnd, str, "Fallout 2 Patcher", MB_OK|MB_ICONINFORMATION);
}


/*resolution
0x000bb118
	00 = 640x480
	0c = 800x600
	18 = 1024x768
	24 = 1280x1024
	30 = 1200x1600

640x48:

800x600:

1024x768:

1280x1024:

1200x1600:

*/