/*
	Author: B.McA (jdoe407)
	Date: March 05, 2005

	Purpose: Generic binary patcher class
	Usage: filePatcher varName( executable, array of patch struct's, # of patches in patch array, output function );
	Todo: Take advantage of the class structure a bit more other then a constructor/destructor
*/

#ifndef _PATCHER_H_
#define _PATCHER_H_

#include <fstream>

struct patch
{
	int offset;
	char *originalData;
	char *patchedData;
	int length;
};

// Decimal -> assembly number
char *numToAsm(unsigned int number);

// checks two NON null-terminated strings
bool binncmp(char *strA, char *strB, int len);

class filePatcher
{
public:
	filePatcher(char *fileName, patch *patchData, int patches, void *pOutputFunction = NULL);
	~filePatcher();
	void applyMorePatches(patch *patchData, int patches, bool bSilent = true);
private:
	static void defaultOutputHandler(char *buff, ...);
	void backupFile();
	void copyFile();
	void determinOutputHandler(void *pOutputFunction);
	void filePatcher::startupPatcher();
	bool makeBackupFile();
	bool replaceData(bool bWriteData, bool bPatchOldFile = false);
	void (*pOutput)(char *, ...);

	bool bDeleteArrays;      // Check if we should delete this array
	bool bDeleteNewFileName; // Check if we should delete this array
	char *_fileName;         // Filename of the unpatched file
	char *_newFileName;      // Filename of the patched file
	int _patches;            // Number of elements in array _patchData
	patch *_patchData;       // Array of patch's containing info on what to patch
	std::fstream _inFile;    // File we read data from
	std::fstream _outFile;   // File we write data to (the patched exe)
};

#endif